;******************************************************************************
;   This file is a basic code template for code generation on the             *
;   PIC12F1822. This file contains the basic code building blocks to build    *
;   upon.                                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:        CHRONOGRAPH SENSOR.asm                           *
;    Date:            JANUARY 26, 2016                                           *
;    File Version:                                                            *
;                                                                             *
;    Author:        Ron Newton                                                *
;    Company:                                                                 *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P12F1822.INC                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;******************************************************************************
;  1	V+
;  2	RA5	OUT
;  3	RA4	IN  C1N1- PIN
;  4	RA3	NC
;  5	RA2	NC
;  6	RA1 NC
;  7	RA0 NC
;  8   	GND
;
;          FILE USED FOR NUTS AND VOLTS 2/10/17                                                                   *
;                                                                             *
;******************************************************************************
;******************************************************************************
;    TAKES 1024 READING AND AVERAGES THEM AND THEN ADDS A FACTOR FOR OFFSET   *
;    USE WDT FOR 1 MINUTE AND RECALCULATES FOR FLUXATIONS
;    Revision History:  DEC 1, 2015                                                      *
;                                                                             *
;******************************************************************************


	list		p=12F1822      ; list directive to define processor
	#include	<p12F1822.inc> ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------    

    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_OFF

;------------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0-15.  Each Bank may contain 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;------------------------------------------------------------------------------

    CBLOCK 0x20 ; Define GPR variable register locations
    
;***** VARIABLE DEFINITIONS
w_temp       	 	
status_temp 	 	
SECOND         		
SECOND_TEN     		
MINUTE         		
DELAY		   		
COUNT		   		
COUNTER1       	
COUNTER2	  	
COUNTER3	   		
COUNTER4
COUNTER5
FRACTION			
ACCcLO				
ACCcMID			    
ACCcHI		    	
ACCaLO			    
ACCaMID		    	
ACCaHI			    
ACCbLO		        
ACCbMID			    
ACCbHI			    
temp		        
ACCdLO		        
ACCdMID			    
ACCdHI
FLAG	
AVG_MID
AVG_LO		    

    ENDC

SAMPLE1        EQU    0x7D        ; Sample user registers (common RAM)
SAMPLE2        EQU    0x7E        ; 
SAMPLE3        EQU    0x7F        ; 

#DEFINE		IN	   	PORTA, 4     ;A/D IN PIN 3
#DEFINE		OUT     PORTA, 5 	 ;PULSE PIN 2

msb_bit equ     7               ;define for bit 7
lsb_bit equ     0               ;define for bit 0

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

    ORG     0x0000            ; processor reset vector
    GOTO    main              ; When using debug header, first inst.
                              ; may be passed over by ICD2.  

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------
;NONE
	

; these first 4 instructions are not required if the internal oscillator is not used
main
		BANKSEL		OSCCON			;CHANGE BANK		
		MOVLW		B'11110000'		;WORD = 1 16 MHZ
		MOVWF		OSCCON			;load
		BANKSEL		PORTA			;CHANGE BANK
		CLRF 		PORTA 			;Init PORTA
		MOVLW 		0Fh 			;Set GP<2:0> to
		BANKSEL		LATA			;CHANGE BANK
		CLRF		LATA			;CLEAR LATA
		BANKSEL		ANSELA			;CHANGE BANK
		MOVLW		B'00010000'		;LOAD ANALOG PIN
		MOVWF		ANSELA			;LOAD
	        BANKSEL		TRISA			;CHANGE BANK
		MOVLW		B'00010000'		;PROGRAM PORTA PORTS
	        MOVWF		TRISA 			;load
	        BANKSEL		OPTION_REG		;CHANGE BANK
		MOVLW		B'10000111'		;SET PRESCALER TO 16 TMRO
		MOVWF		OPTION_REG		;LOAD
		BANKSEL		ADCON0			;CHANGE BANK
		MOVLW		B'10001101'		;A/D ON 
		MOVWF		ADCON0			;LOAD
		BANKSEL		ADCON1			;CHANGE BANK
		MOVLW		B'10100011'		;RIGHT JUSTIFIED INTERNAL VOLTAGE REF
		MOVWF		ADCON1			;LOAD
		BANKSEL		FVRCON			;CHANGE BANK
		MOVLW		B'10001100'		;VOLTAGE REF ON  1.024VOLTS FOR DAC 0100 1.024 1000 2.048, 1100 4.096
		MOVWF		FVRCON			;LOAD  now loaded with 4.096
		BANKSEL		DACCON0
		MOVLW		B'11001000'		;DAC ON, DAC POS, DAC DISCONNECTED FVR BUFFER2 OUTPUT

;**********************************************
;***********************VOLTAGE CHANGE***********************
		MOVWF		DACCON0
		MOVLW		.20			;10
		MOVWF		DACCON1			;.32VOLTS 20 = 2.56

;**********************************************
;***********************************************
		BANKSEL		INTCON			;CHANGE BANK
		CLRF		INTCON
		BANKSEL		PORTA			;CHANGE BANK
		BCF		OUT		;SET OUT  (PULSE OFF)
		BANKSEL		CM1CON0
		MOVLW		B'10000100'	;ON HIGHSPEED
		MOVWF		CM1CON0
		MOVLW		B'00010001'	;C1N1- RA4 INPUT
		MOVWF		CM1CON1

;**************************************************************
TESTING	

;**************************************************************
HERE   
		BANKSEL		CM1CON0
		BTFSC		CM1CON0,6		;BTFSS CORRECT OLD SET VOLTAGE ABOVE REF
		GOTO		CLICK	
	  	GOTO		HERE			;NOT ABOVE DO AGAIN

;*********************************************
CLICK   
		BANKSEL		PORTA			;CHANGE BANK
		BSF		OUT			;PULSE OUT
		MOVLW		.22
		MOVWF		COUNTER1
		DECFSZ		COUNT,1			;22 MILLI SEC DELAY
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		BCF			OUT
;		MOVLW		.11
;		MOVWF		COUNTER1
;		DECFSZ		COUNT,1			;2 MSEC DELAY
;		GOTO		$-1
;		DECFSZ		COUNTER1,1
;		GOTO		$-3
		GOTO		HERE



		END                       ; directive 'end of program'    
    
    
    
    
    
    



