;******************************************************************************
;   This file is a basic code template for code generation on the             *
;   PIC16F916. This file contains the basic code building blocks to build     *
;   upon.                                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:        CHRONOGRAPH.asm                                         *
;    Date:            FEBRUARY 21, 2015                                        *
;    File Version:    INTERRUPT FOR TMR01     
;	 used for N&V  2/10/17                            			              *
;                                                              *
;    Author:       Ron Newton                                                 *
;    Company:                                                                 *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P16F916.INC                                              *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:   File used for N&V 2-17                                                                *
;                                                                             *
;******************************************************************************
;                                                                             *
;                                    *
;******************************************************************************


	list		p=16f916      ; list directive to define processor
	#include	<p16f916.inc> ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------    

;1	MCLR		NC
;2	RA0		LED 1	
;3	RA1		WHITE SENSOR 1
;4	RA2		YELLOW SENSOR 2
;5	RA3		NC
;6	RA4		NC
;7	RA5		RS
;8	GND
;9	RA7		NC
;10	RA6		LED 2
;11	RC0		D0
;12	RC1		D1
;13	RC2		D2
;14	RC3		D3
;15	RC4		D4
;16	RC5		D5
;17	RC6		D6
;18	RC7		D7
;19	GND
;20	V POS
;21	RB0		E
;22	RB1		R/W
;23	RB2		NC
;24	RB3		NC
;25	RB4		SWITCH 2
;26	RB5		NC
;27	RB6		NC
;28	RB7		SWITCH 1

	
	__CONFIG _FCMEN_OFF & _IESO_ON & _CP_OFF & _WDT_OFF & _BOD_OFF & _MCLRE_ON & _PWRTE_OFF & _INTRC_OSC_NOCLKOUT

;------------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0,1,2, and 3.  Each Bank contains 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;------------------------------------------------------------------------------

;       Bit Equates 
;****************************************************************************
DONE    equ     7               ;done measuring flag
EMPTY   equ     5               ;flag if component is connected
V1      equ     5               ;power for C reference ckt
msb_bit equ     7               ;define for bit 7
lsb_bit equ     0               ;define for bit 0
time	equ	3
sign   equ	4

    CBLOCK 0x20 ; Define GPR variable register locations


;****************************************************************************
;       User Registers
;****************************************************************************
;       Bank 0


SHOT
count  	            ;RS232 register
txreg  	            ;RS232 data register
delay  	            ;RS232 delay register
offset  	        ;table position register
msb    	            ;general delay register
lsb    	            ;general delay register
COUNT
COUNT1
COUNT2
TEMP
TEMP1
S1_TIMER_LO
S2_TIMER_LO
S3_TIMER_LO
S4_TIMER_LO
S5_TIMER_LO
S6_TIMER_LO
S7_TIMER_LO
S8_TIMER_LO
S9_TIMER_LO
S10_TIMER_LO

S1_TIMER_HI
S2_TIMER_HI
S3_TIMER_HI
S4_TIMER_HI
S5_TIMER_HI
S6_TIMER_HI
S7_TIMER_HI
S8_TIMER_HI
S9_TIMER_HI
S10_TIMER_HI


S1_TIMER_MID
S2_TIMER_MID
S3_TIMER_MID
S4_TIMER_MID
S5_TIMER_MID
S6_TIMER_MID
S7_TIMER_MID
S8_TIMER_MID
S9_TIMER_MID
S10_TIMER_MID


AVG_TIMER_LO
AVG_TIMER_MID
AVG_TIMER_HI
HOLDTIMER_HI
FLAG

;		DOG REGISTERS
DATA1
COUNTER0
COUNTER1
ONES
TENS
HUNDREDS
THOUSANDS
DIGIT_1
DIGIT_2
DIGIT_3
DIGIT_4
DIGIT_5
;       Math related registers
ACCaHI  	           ;24-Bit accumulator a
ACCaMID 	
ACCaLO  	
ACCbHI  	           ;24-Bit accumulator b
ACCbMID 	
ACCbLO  	
ACCcHI  	           ;24-Bit accumulator c
ACCcMID 	
ACCcLO  	
ACCdHI  	           ;24-Bit accumulator d
ACCdMID 	
ACCdLO  	
TENK					;10,000
temp   	           		;temporary storage
TEMPL
TEMPH
MAX_SHOT
MIN_SHOT
MAX_H
MAX_L
MAX_TIMER_HI
MIN_H
MIN_L
MIN_TIMER_HI
HOLDL
HOLDH
SHOT1
NUMBER
TIMER_LO
TIMER_MID
TIMER_HI
    ENDC

W_TEMP         EQU    0x7D        ; w register for context saving (ACCESS)
STATUS_TEMP    EQU    0x7E        ; status used for context saving (ACCESS)
PCLATH_TEMP    EQU    0x7F        ; context saving of PC (ACCESS) 

#DEFINE		RS				PORTA,5		;RS OUT
#DEFINE		LED2				PORTA,6
#DEFINE		LED1				PORTA,0
#DEFINE		PULSE_OUT			PORTA,7
#DEFINE		E				PORTB,0		;E OUT
#DEFINE		RW				PORTB,1		;RW OUT
#DEFINE		SW2				PORTB,4
#DEFINE		SENSOR2				PORTA,2
#DEFINE		SENSOR1				PORTA,1
#DEFINE		SW1			    	PORTB,7

;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; The 16F916 has 256 bytes of non-volatile EEPROM, STARTing at address 0x2100
; 
;------------------------------------------------------------------------------

DATAEE    ORG  0x2100
;		DE	0x00, 0x01, 0x02, 0x03


;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

    ORG     0x0000            ; processor reset vector
    PAGESEL init
        GOTO    init

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------
    ORG      0x0004
    ; Interrupt context saving
    MOVWF    W_TEMP           ; save off current W register contents
    MOVF     STATUS,W         ; move status register into W register
    MOVWF    STATUS_TEMP      ; save off contents of STATUS register
    MOVF     PCLATH,W         ; move pclath register into w register
    MOVWF    PCLATH_TEMP      ; save off contents of PCLATH register

;------------------------------------------------------------------------------

	INCF	TIMER_HI,1	;TMR1 OVERFLOW
	MOVLW	01EH		;ONE SECOND CHECK OR FALSE TRIGGERING
	SUBWF	TIMER_HI,0
	BTFSC	3,0
	BSF		FLAG,3	;TRUE SET FLAG 3
	
	BCF		PIR1,0	;CLEAR INTERRUPT



;------------------------------------------------------------------------------

    ; Interrupt context restoring
    MOVF    PCLATH_TEMP,W     ; retrieve copy of PCLATH register
    MOVWF   PCLATH            ; restore pre-isr PCLATH register contents
    MOVF    STATUS_TEMP,W     ; retrieve copy of STATUS register
    MOVWF   STATUS            ; restore pre-isr STATUS register contents
    SWAPF   W_TEMP,F
    SWAPF   W_TEMP,W          ; restore pre-isr W register contents
    RETFIE                    ; return from interrupt

    
;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

init
		BANKSEL OSCCON
		MOVLW	B'01110000'		;8MHZ
		MOVWF	OSCCON
		BANKSEL	PORTA			;change banks
		clrf    PORTA			;clear PORTA, PORTB and PORTC
		clrf    PORTB
		clrf	PORTC
		BANKSEL	CMCON0			;CLEAR CMCON0
		MOVLW	.255
		
		MOVWF	CMCON0
		BANKSEL	ANSEL
		CLRF	ANSEL			;CLEAR ANSEL
		BANKSEL	OPTION_REG		;change banks
		MOVLW	B'00000111'		;TMR0 prescalar 1:256  10 SECONDS
		MOVWF   OPTION_REG
		BANKSEL	TRISA			;change banks
		MOVLW	B'00000110'		;RA1 AND RA2 INPUT
		MOVWF	TRISA			;
		MOVLW	B'10010000'
		MOVWF	TRISB			;MAKE PORT 7-4 INPUT FOR SWITCH AND DETECTORS
		CLRF	TRISC			;PORTC all outputs
		BANKSEL	WPUB
		MOVLW	B'10010000'
		MOVWF	WPUB			;USE PULL UP FOR SWITCHES
		MOVLW	B'11110000'
		MOVWF	IOCB			;ALLOW INTERRUPTS
		BANKSEL	LCDCON
		CLRF	LCDCON			;CLEAR LCD CON
		BANKSEL	PORTA

		CLRF	T1CON			;TURN OFF AND USE CLOCK
		CALL	CLEAR_REG
		MOVLW	031H
		MOVWF	SHOT			;DISPLAY NUMBER OF SHOT
		CLRF	FLAG
		CALL	DOG
;********************TEST AREA*********************
TEST		



;**************************************************
START		BTFSS	SW1			;HOLD SW1 WHEN POWERING UP FOR VOLTAGE
		GOTO	VOLTAGE			;PUSHED JUMP TO VOLTAGE
		BTFSC	SW2			;HOLD SW2 WHEN POWERING UP FOR TESTING
		GOTO	BEGIN			;NOT PUSHED JUMP TO BEGIN

		CALL	DOG
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	054H			;TEST MODE
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	04DH			;M
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	044H			;D
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND

S1		BTFSC	SENSOR1			;CHECK SENSOR 1 FOR PULSE
		GOTO	A			;JUMP
		BCF	LED1			;TURN OFF LED1
		GOTO	AB			;JUMP
A		BSF	LED1			;TURN ON LED1
AB		BTFSC	SENSOR2			;CHECK SENSOR 2
		GOTO	AA			;JUMP
		BCF	LED2			;TURN OFF LED 2
		GOTO	S1			;DO AGAIN
AA		BSF	LED2			;TURN ON LED 2
		GOTO	S1			;DO AGAIN

		
VOLTAGE		BSF	FLAG,2			;SET FLAG 2
		BANKSEL	ADCON1
		MOVLW	B'01110000'		;TURN A/D
		MOVWF	ADCON1
		BANKSEL	TRISA
		BSF	TRISA,1			;MAKE RA 1 INPUT
		BSF	TRISA,2			;MAKE RA 2 INPUT
		BANKSEL	ANSEL
		BSF	ANSEL,1			;TURN ON A/D
		BSF	ANSEL,2			;TURN ON A/D
		BANKSEL	PORTA
		PAGESEL	VOLT1			;CHANGE PAGE
		GOTO	VOLT1			;DO AGAIN

		
;*****************************END OF TESTING************

BEGIN		BTFSC	SENSOR1		    ;MAKING SURE SENSORS ARE CLEAR
		GOTO	$-1
		BTFSC	SENSOR2
		GOTO	$-1
		BCF	LED1
		BCF	LED2

B3		CALL	DOG
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;SHOOTING
		CALL	SEND
		MOVLW	048H			;H
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	049H			;I
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	047H			;G
		CALL	SEND
		CALL	vlong
		BTFSS	SW2			;ANTIBOUNCE
		GOTO	$-1

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S2 FOR STATS
		CALL	SEND	
		MOVLW	032H			;2
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	046H			;F
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND

;**************************************************************************************************
		BANKSEL	PORTA
		MOVLW	.0		;CLEAR DOES NOT WORK
		MOVWF	TMR1L
		MOVWF	TMR1H
		CLRF	TIMER_HI
		BCF	FLAG,3		;CLEAR FLAG 3
		BCF	PIR1,0		;CLEAR PIR1,0 FLAG
		BANKSEL	PIE1
		BSF	PIE1,0		;ALLOW PIR1 INTERRUPT
		BANKSEL	PORTA
		MOVLW	B'11000000'	;TURN ON INTERRUPT
		MOVWF	INTCON	
		BTFSS	SW2		;CHECK SW2 FOR GOING TO STATS
		GOTO	STATS
;*****************************
;*****************************
		BTFSS	SENSOR1			;CHECKING SENSOR 1 FOR PULSE
		GOTO	$-3
		BSF	T1CON,0			;TURN ON TIMER1
BB		BTFSS	FLAG,3			;CHECK FOR OVERFLOW
		GOTO	EE			;JUMP
		BSF	LED1			;OVERM 1 SECOND TURN ON LED 1
		BTFSS	SW2			;IN CASE LED  ONE IS TURNED ON
		GOTO	BEGIN			;RESET IF LED IS ON
		GOTO	$-3			;HOLD UNTIL SW2 IS PUSHED
EE		BTFSS	SENSOR2			;CHECK SENSOR 2 FOR PULSE
		GOTO	BB			;JUMP
		BCF	T1CON,0			;TURN OFF TIMER1
;********************************
;********************************

		BSF	LED2			;TURN ON BOTH LEDS
		BSF	LED1
		CLRF	INTCON			;DISALLOW INTERRUPTS
		BANKSEL	PIE1			;TURN OFF TMR1H INTERRUPT
		CLRF	PIE1
		BANKSEL	PORTA
		CALL	CLEAR			;CLEAR MATH REGISTERS
		MOVF	TMR1L,0			;MOVING TO PREVENT CHANGE OF TMR1L AND TMR1H
		MOVWF	TIMER_LO
		MOVF	TMR1H,0			
		MOVWF	TIMER_MID



		MOVF	TIMER_LO,0		;LOADING MATH REGISTERS
		MOVWF	ACCaLO
		MOVF	TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	TIMER_HI,0
		MOVWF	ACCaHI
		CALL	FPS			;CALULATE FPS

		CALL	DOG
		MOVLW	046H			;FPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		BTFSS	FLAG,1
		GOTO	H1
		MOVLW	02EH			;DECIMAL
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND
H1		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	023H			;#
		CALL	SEND
		MOVLW	03AH			;CHECK FOR 10
		SUBWF	SHOT,0
		BTFSS	3,2
		GOTO	BA	
		MOVLW	031H
		CALL	SEND
		MOVLW	030H
		CALL	SEND
		GOTO	BAA	
BA		MOVF	SHOT,0
		CALL	SEND
BAA		CALL	CLEAR
		MOVF	TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	TIMER_HI,0
		MOVWF	ACCaHI
		CALL	MPS

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK	
		MOVLW	04DH			;MPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVLW	02EH
		CALL	SEND			;.
		MOVF	DIGIT_1,0
		CALL	SEND
B4		BTFSC	SW1
		GOTO	B5
		GOTO	STORE
B5		BTFSC	SW2
		GOTO	B4
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1	
		GOTO	BEGIN				;DO OVER

;******************END OF BEGINING**************


	
AVERAGE		CALL	CLEAR		    ;CLEAR MATH REGISTERS
		MOVF	S10_TIMER_LO,0	    ;LOADING INTO MATH REGISTERS
		MOVWF	ACCbLO
		MOVF	S10_TIMER_MID,0
		MOVWF	ACCbMID
		MOVF	S10_TIMER_HI,0
		MOVWF	ACCbHI
		MOVF	S9_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S9_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S9_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24	
		MOVF	S8_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S8_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S8_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24		
		MOVF	S7_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S7_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S7_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24		    ;ADD
		MOVF	S6_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S6_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S6_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24
		MOVF	S5_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S5_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S5_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24
		MOVF	S4_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S4_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S4_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24
		MOVF	S3_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S3_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S3_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24
		MOVF	S2_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S2_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S2_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24
		MOVF	S1_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	S1_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	S1_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	Add24
TAKE_AVG	MOVF	ACCbLO,0		;SWAPING OUT REGISTERS
		MOVWF	ACCcLO
		MOVF	ACCbMID,0
		MOVWF	ACCcMID
		MOVF	ACCbHI,0
		MOVWF	ACCcHI
		CLRF	ACCbLO			;CLEARING B REGISTERS
		CLRF	ACCbMID
		CLRF	ACCbHI
		CLRF	ACCaMID
		CLRF	ACCaHI
		MOVF	NUMBER,0
		MOVWF	ACCaLO
		CALL	Div24
		MOVF	ACCcLO,0
		MOVWF	AVG_TIMER_LO
		MOVF	ACCcMID,0
		MOVWF	AVG_TIMER_MID
		MOVF	ACCcHI,0
		MOVWF	AVG_TIMER_HI
		RETURN
;************************************************************



DISPLAY_AVERAGE
		MOVLW	.0					;CHECKING FOR 0s OR NOTHING LOADED YET
		SUBWF	AVG_TIMER_LO,0
		BTFSC	3,2
		GOTO	DA1
		GOTO	DA3
DA1		MOVLW	.0
		SUBWF	AVG_TIMER_MID,0
		BTFSC	3,2
		GOTO	DA2
		GOTO	DA3
DA2		MOVLW	.0
		SUBWF	AVG_TIMER_HI,0
		BTFSC	3,2
		GOTO	DA2A
		GOTO	DA3
DA2A		BSF	FLAG,0
		RETURN

DA3		CALL	CLEAR
		MOVF	AVG_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	AVG_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	AVG_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	FPS
		CALL	DOG
		MOVLW	041H			;AVG
		CALL	SEND
		MOVLW	056H			;V
		CALL	SEND	
		MOVLW	047H			;G
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	046H			;FPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		BTFSS	FLAG,1
		GOTO	H2
		MOVLW	02EH			;DECIMAL
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND

H2		CALL	CLEAR
		MOVF	AVG_TIMER_LO,0
		MOVWF	ACCaLO
		MOVF	AVG_TIMER_MID,0
		MOVWF	ACCaMID
		MOVF	AVG_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	MPS

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK	
		MOVLW	041H			;AVG
		CALL	SEND
		MOVLW	056H			;V
		CALL	SEND	
		MOVLW	047H			;G
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	04DH			;MPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVLW	02EH			;DECIMAL
		CALL	SEND			;.
		MOVF	DIGIT_1,0
		CALL	SEND
		RETURN

;**********************************************STORING******************************

STORE		CALL	CLEAR		;CLEAR MATH REGISTERS
		MOVLW	031H
		SUBWF	SHOT,0
		BTFSS	3,2		;IF EQUAL
		GOTO	ST2
		MOVF	TIMER_LO,0
		MOVWF	S1_TIMER_LO
		MOVWF	MAX_L
		MOVWF	MIN_L
		MOVF	TIMER_MID,0
		MOVWF	S1_TIMER_MID
		MOVWF	MAX_H
		MOVWF	MIN_H
		MOVF	TIMER_HI,0
		MOVWF	S1_TIMER_HI
		MOVWF	MAX_TIMER_HI
		MOVWF	MIN_TIMER_HI
		MOVLW	031H
		MOVWF	MAX_SHOT
		MOVWF	MIN_SHOT
		MOVLW	.1
		MOVWF	NUMBER
		INCF	SHOT,1
		GOTO	BEGIN

ST2		MOVLW	032H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST3
		MOVF	TIMER_LO,0
		MOVWF	S2_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S2_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S2_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN
	
ST3		MOVLW	033H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST4
		MOVF	TIMER_LO,0
		MOVWF	S3_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S3_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S3_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF    NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST4		MOVLW	034H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST5
		MOVF	TIMER_LO,0
		MOVWF	S4_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S4_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S4_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST5		MOVLW	035H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST6
		MOVF	TIMER_LO,0
		MOVWF	S5_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S5_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S5_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST6		MOVLW	036H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST7
		MOVF	TIMER_LO,0
		MOVWF	S6_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S6_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S6_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST7		MOVLW	037H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST8
		MOVF	TIMER_LO,0
		MOVWF	S7_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S7_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S7_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST8		MOVLW	038H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST9
		MOVF	TIMER_LO,0
		MOVWF	S8_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S8_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S8_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST9		MOVLW	039H
		SUBWF	SHOT,0
		BTFSS	3,2				;IF EQUAL
		GOTO	ST10
		MOVF	TIMER_LO,0
		MOVWF	S9_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S9_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S9_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1
		INCF	SHOT,1
		GOTO	BEGIN

ST10	MOVF	TIMER_LO,0
		MOVWF	S10_TIMER_LO
		MOVWF	TEMPL
		MOVF	TIMER_MID,0
		MOVWF	S10_TIMER_MID
		MOVWF	TEMPH
		MOVF	TIMER_HI,0
		MOVWF	S10_TIMER_HI
		MOVWF	TIMER_HI
		CALL	MAX_MIN
		INCF 	NUMBER,1

		CALL	DOG
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S1 FOR STATS
		CALL	SEND
		MOVLW	031H			;1
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	046H			;F
		CALL	SEND
		MOVLW	04FH			;0
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;SW TO CLEAR
		CALL	SEND	
		MOVLW	032H			;2
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	043H			;C
		CALL	SEND	
		MOVLW	04CH			;L
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND	
		BTFSS	SW1			;CHECKING SWITCHES
		GOTO	STATS
		BTFSC	SW2
		GOTO	$-3
		CALL	CLEAR_REG		;CLEARING REGISTERS
		CALL	vlong
		BTFSC	SW2			;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
;**********************STATS***************************
STATS		BCF	LED1		;TURN OFF LEDS
		BCF	LED2
		CALL	vlong
		BTFSS	SW1
		GOTO	$-1
		BTFSS	SW2
		GOTO	$-1
		CALL	DOG
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		MOVLW	053H			;S1 TO ADVANCE
		CALL	SEND
		MOVLW	031H			;1
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	054H			;TO
		CALL	SEND
		MOVLW	04FH			;0
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	044H			;D
		CALL	SEND
		MOVLW	056H			;V
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND
		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S2 TO SHOOT
		CALL	SEND	
		MOVLW	032H			;2
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	048H			;H
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND	
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_0
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN

DIS_0		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		CALL	AVERAGE
		CALL	DISPLAY_AVERAGE
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1
		GOTO	DIS_00
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN

DIS_00	CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		CALL	DISPLAY_MAX
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_000
		BTFSC	SW2
		GOTO	$-3
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN

DIS_000 CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		CALL	DISPLAY_MIN
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_1
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN

DIS_1		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S1_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S1_TIMER_MID,0
		MOVWF	HOLDH
		MOVF	S1_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	031H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_2
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_2		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S2_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S2_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S2_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	032H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_3
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_3		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S3_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S3_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S3_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	033H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_4
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_4		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S4_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S4_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S4_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	034H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_5
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_5		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S5_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S5_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S5_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	035H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_6
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_6		CALL	vlong
		BTFSS	SW1
		GOTO	$-1
		MOVF	S6_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S6_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S6_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	036H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_7
		BTFSC	SW2
		GOTO	$-3
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_7		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S7_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S7_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S7_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	037H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_8
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_8		CALL	vlong
		BTFSS	SW1
		GOTO	$-1
		MOVF	S8_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S8_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S8_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	038H
		MOVWF	SHOT1
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_9
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_9		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S9_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S9_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S9_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	039H
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_10
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN
DIS_10		CALL	vlong
		BTFSS	SW1				;ANTIBOUNCE
		GOTO	$-1
		MOVF	S10_TIMER_LO,0	
		MOVWF	HOLDL
		MOVF	S10_TIMER_MID,0
		MOVWF	HOLDH
		MOVWF	HOLDH
		MOVF	S10_TIMER_HI,0
		MOVWF	HOLDTIMER_HI
		MOVLW	03AH
		MOVWF	SHOT1
		CALL	DISPLAY_REGISTERS
		BTFSC	FLAG,0
		GOTO	DIS_11
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	DIS_11
		BTFSC	SW2
		GOTO	$-3
		CALL	vlong
		BTFSS	SW2				;ANTIBOUNCE
		GOTO	$-1
		GOTO	BEGIN

DIS_11	BCF		FLAG,0
		BTFSS	SW1
		GOTO	$-1
		CALL	DOG
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	045H				;END OF FILE
		CALL	SEND
		MOVLW	04EH				;N
		CALL	SEND
		MOVLW	044H				;D
		CALL	SEND
		MOVLW	020H				;SPACE
		CALL	SEND
		MOVLW	04FH				;O
		CALL	SEND
		MOVLW	046H				;F
		CALL	SEND
		MOVLW	020H				;SPACE
		CALL	SEND
		MOVLW	046H				;F
		CALL	SEND
		MOVLW	049H				;I
		CALL	SEND
		MOVLW	04CH				;L
		CALL	SEND
		MOVLW	045H				;E
		CALL	SEND

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK
		MOVLW	053H			;S1=BACK S2=CLEAR 
		CALL	SEND	
		MOVLW	031H			;1
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND	
		MOVLW	042H			;B
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	04BH			;K
		CALL	SEND
		MOVLW	041H		
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	032H			;2
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND	
		MOVLW	043H			;C
		CALL	SEND
		MOVLW	04CH			;L
		CALL	SEND
		MOVLW	045H			;E
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND
		MOVLW	.6
		MOVWF	COUNT2
		CALL	vlong
		BTFSS	SW1				;CHECKING SWITCHES
		GOTO	BEGIN
DIS_12		BTFSC	SW2
		GOTO	$-3
		DECFSZ	COUNT,1
		GOTO	DIS_12
		DECFSZ	COUNT1,1
		GOTO	$-3
		DECFSZ	COUNT2,1
		GOTO	$-5
		CALL	CLEAR_REG		;CLEARING REGISTERS
		GOTO	BEGIN
		RETURN


;**************************DISPLAY REGISTERS*********************8

DISPLAY_REGISTERS
		MOVLW	.0
		SUBWF	HOLDL,0
		BTFSC	3,2
		GOTO	DR1
		GOTO	DR3
DR1		MOVLW	.0
		SUBWF	HOLDH,0
		BTFSC	3,2
		GOTO	DR2
		GOTO	DR3
DR2		BSF	FLAG,0
		RETURN
DR3		CALL	CLEAR
		MOVF	HOLDL,0
		MOVWF	ACCaLO
		MOVF	HOLDH,0
		MOVWF	ACCaMID
		MOVF	HOLDTIMER_HI,0
		MOVWF	ACCaHI
		CALL	FPS
		CALL	DOG
		MOVLW	046H			;FPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	02H			;SPACE
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	02H			;SPACE
		CALL	SEND
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		BTFSS	FLAG,1
		GOTO	H3
		MOVLW	02EH			;DECIMAL
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND
H3		MOVLW	02H			;SPACE
		CALL	SEND
		MOVLW	02H			;SPACE
		CALL	SEND
		MOVLW	03AH			;CHECK FOR 10
		SUBWF	SHOT1,0
		BTFSS	3,2
		GOTO	DIS_BA
		MOVLW	023H			;#
		CALL	SEND	
		MOVLW	031H
		CALL	SEND
		MOVLW	030H
		CALL	SEND
		GOTO	DIS_BAA	

DIS_BA		MOVLW	023H			;#
		CALL	SEND
		MOVF	SHOT1,0
		CALL	SEND

	
DIS_BAA
		CALL	CLEAR
		MOVF	HOLDL,0
		MOVWF	ACCaLO
		MOVF	HOLDH,0
		MOVWF	ACCaMID
		MOVF	HOLDTIMER_HI,0
		MOVWF	ACCaHI
		CALL	MPS

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK
		MOVLW	04DH			;MPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVLW	02EH
		CALL	SEND			;.
		MOVF	DIGIT_1,0
		CALL	SEND
		RETURN
;********************DISPLAY MAX **********************************************************************8
DISPLAY_MAX
		CALL	CLEAR
		MOVF	MAX_L,0
		MOVWF	ACCaLO
		MOVF	MAX_H,0
		MOVWF	ACCaMID
		MOVF	MAX_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	FPS
		CALL	DOG
		MOVLW	04DH			;MAX
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	058H			;X
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	046H			;FPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	020H
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H
		CALL	SEND
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		BTFSS	FLAG,1
		GOTO	H4
		
		MOVLW	02EH		    ;DECIMAL
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND

H4		CALL	CLEAR
		MOVF	MAX_L,0
		MOVWF	ACCaLO
		MOVF	MAX_H,0
		MOVWF	ACCaMID
		MOVF	MAX_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	MPS

		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK
		MOVLW	04DH			;MAX
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	058H			;X
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	04DH			;MPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	03DH			;=
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVLW	02EH
		CALL	SEND			;.
		MOVF	DIGIT_1,0
		CALL	SEND
		MOVLW	03AH			;CHECK FOR 10
		SUBWF	SHOT,0
		BTFSS	3,2
		GOTO	MAX_BA	
		MOVLW	023H			;#
		CALL	SEND
		MOVLW	031H
		CALL	SEND
		MOVLW	030H
		CALL	SEND
		GOTO	MAX_BAA	
MAX_BA		MOVLW	20H
		CALL	SEND
		MOVLW	023H			;#
		CALL	SEND
		MOVF	MIN_SHOT,0
		CALL	SEND
MAX_BAA		RETURN

;*****************************************************
DISPLAY_MIN
		CALL	CLEAR
		MOVF	MIN_L,0
		MOVWF	ACCaLO
		MOVF	MIN_H,0
		MOVWF	ACCaMID
		MOVF	MIN_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	FPS
		CALL	DOG
		MOVLW	04DH			;MIN
		CALL	SEND
		MOVLW	049H			;I
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	046H			;FPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	020H
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND
		MOVLW	020H
		CALL	SEND
		MOVF	DIGIT_5,0
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		BTFSS	FLAG,1
		GOTO	H5		
		MOVLW	02EH			;DECIMAL
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND
		
H5		CALL	CLEAR
		MOVF	MIN_L,0
		MOVWF	ACCaLO
		MOVF	MIN_H,0
		MOVWF	ACCaMID
		MOVF	MIN_TIMER_HI,0
		MOVWF	ACCaHI
		CALL	MPS

		MOVLW		B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF		DATA1
		CALL		WRINS_CHK
		MOVLW	04DH			;MIN
		CALL	SEND
		MOVLW	049H			;I
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND	
		MOVLW	04DH			;MPS = 
		CALL	SEND	
		MOVLW	050H			;P
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND	
		MOVLW	03DH			;=
		CALL	SEND
		MOVF	DIGIT_4,0
		CALL	SEND
		MOVF	DIGIT_3,0
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVLW	02EH
		CALL	SEND			;.
		MOVF	DIGIT_1,0
		CALL	SEND
		MOVLW	03AH			;CHECK FOR 10
		SUBWF	SHOT,0
		BTFSS	3,2
		GOTO	MIN_BA	
		MOVLW	023H			;#
		CALL	SEND
		MOVLW	031H
		CALL	SEND
		MOVLW	030H
		CALL	SEND
		GOTO	MIN_BAA	
MIN_BA		MOVLW	20H
		CALL	SEND
		MOVLW	023H			;#
		CALL	SEND
		MOVF	MIN_SHOT,0
		CALL	SEND
MIN_BAA		RETURN


;**********************************MAX_MIN****************
MAX_MIN		CALL	CLEAR
		MOVF	MAX_L,0
		MOVWF	ACCaLO
		MOVF	MAX_H,0
		MOVWF	ACCaMID
		MOVF	MAX_TIMER_HI,0
		MOVWF	ACCaHI
		MOVF	TEMPL,0
		MOVWF	ACCbLO
		MOVF	TEMPH,0
		MOVWF	ACCbMID
		MOVF	TIMER_HI,0
		MOVWF	ACCbHI
		CALL	Sub24
		MOVLW	0FFH
		SUBWF	ACCbHI,0
		BTFSC	3,2
		GOTO	MM2
		MOVF	TEMPL,0
		MOVWF	MAX_L
		MOVF	TEMPH,0
		MOVWF	MAX_H
		MOVF	SHOT,0
		MOVWF	MAX_SHOT

MM2		CALL	CLEAR
		MOVF	MIN_L,0
		MOVWF	ACCaLO
		MOVF	MIN_H,0
		MOVWF	ACCaMID
		MOVF	MIN_TIMER_HI,0
		MOVWF	ACCaHI
		MOVF	TEMPL,0
		MOVWF	ACCbLO
		MOVF	TEMPH,0
		MOVWF	ACCbMID
		MOVF	TIMER_HI,0
		MOVWF	ACCbHI
		CALL	Sub24
		MOVLW	0FFH
		BTFSS	3,2
		GOTO	MM3
		RETURN
MM3		MOVF	TEMPL,0
		MOVWF	MIN_L
		MOVF	TEMPH,0
		MOVWF	MIN_H
		MOVF	TIMER_HI,0
		MOVWF	MIN_TIMER_HI
		MOVF	SHOT,0
		MOVWF	MIN_SHOT
		RETURN
	
		
FPS		BCF	FLAG,1
		MOVLW	0H			;DIVIDEND = 20,000,000 ONE DECIMAL
		MOVWF	ACCcLO
		MOVLW	02DH
		MOVWF	ACCcMID
		MOVLW	031H
		MOVWF	ACCcHI
		MOVLW	01H
		MOVWF	ACCbLO
		CALL	Div24
		CALL	DECIMAL
		MOVLW	.0^0FFH
		ADDWF	TENK,0
		BTFSS	3,0
		BSF	FLAG,1
		CALL	DISPLAYING
		RETURN
		
MPS		MOVLW	098H			;DIVIDEND = 6,096,000 EXTRA 0 FOR DECIMAL
		MOVWF	ACCcLO
		MOVLW	0AH
		MOVWF	ACCcMID
		MOVLW	05DH
		MOVWF	ACCcHI
		CALL	Div24
		CALL	DECIMAL
		CALL	DISPLAYING
		RETURN
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Delay Routines                                                     -*
;*--------------------------------------------------------------------------*
;****************************************************************************
vlong   movlw   0xff            ;very long delay, approx 200msec
        movwf   msb
        goto    d1
delay20                         ;20 msec delay
        movlw   .26
        movwf   msb
        goto    d1
delay16                         ;16 msec delay
        movlw   .21
        movwf   msb
        goto    d1
delay1                          ;approx 750nsec delay
        movlw   .1      
        movwf   msb
d1      movlw   0xff
        movwf   lsb
d2      decfsz  lsb,F
        goto    d2
        decfsz  msb,F
        goto    d1
        return
;****************************************	

CLEAR	
		CLRF	ACCaLO
		CLRF	ACCaMID
		CLRF	ACCaHI
		CLRF	ACCbLO
		CLRF	ACCbMID
		CLRF	ACCbHI
		CLRF	ACCcLO
		CLRF	ACCcMID
		CLRF	ACCcHI
		CLRF	ACCdLO
		CLRF	ACCdMID
		CLRF	ACCdHI
		RETURN
;---------------------------CONVERSION FROM HEXADECIMAL TO DECIMAL---------------------------------------------------
;*		USES	ACCcLO AND ACCcMID
;*		RESULTS IN TENK (TENS OF THOUSANDS) THOUSANDS, HUNDREDS,TENS AND ONES
;*----------------------------------------------------------------------------------

DECIMAL											
		BANKSEL		PORTA			;change banks
		SWAPF		ACCcMID,W
		ANDLW		0FH
		ADDLW		0F0H
		MOVWF		THOUSANDS
		ADDWF		THOUSANDS,F
		ADDLW		0E2H
		MOVWF		HUNDREDS
		ADDLW		32H
		MOVWF		ONES

		MOVF		ACCcMID,0
		ANDLW		0FH
		ADDWF		HUNDREDS,F
		ADDWF		HUNDREDS,F
		ADDWF		ONES,F
		ADDLW		0E9H
		MOVWF		TENS
		ADDWF		TENS,F
		ADDWF		TENS,F

		SWAPF		ACCcLO,W
		ANDLW		0FH
		ADDWF		TENS,F
		ADDWF		ONES,F

		RLF			TENS,F
		RLF			ONES,F
		COMF		ONES,F
		RLF			ONES,F
			
		MOVF		ACCcLO,W
		ANDLW		0FH
		ADDWF		ONES,F
		RLF			THOUSANDS,F
		
		MOVLW		07H
		MOVWF		TENK

		MOVLW		0AH
LB1		ADDWF		ONES,F
		DECF		TENS,F
		BTFSS		3,0
		GOTO		LB1
LB2		ADDWF		TENS,F
		DECF		HUNDREDS,F
		BTFSS		3,0
		GOTO		LB2

LB3		ADDWF		HUNDREDS,F
		DECF		THOUSANDS,F
		BTFSS		3,0
		GOTO		LB3

LB4		ADDWF		THOUSANDS,F
		DECF		TENK,F
		BTFSS		3,0
		GOTO		LB4

		RETURN
;------------------------------------------------------------------------------
;************************DOG DELAYS FOR PROGRAMMING LCD******************
SEND		MOVWF		DATA1
		CALL		WRDATA_CHK
		CALL		DELAY30uS
		RETURN
WRDATA_CHK
		CALL		CHK_BUSY
		BSF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTC
		BCF			E
		MOVLW		.255
		MOVWF		PORTC
		RETURN
WRINS_CHK
		CALL		CHK_BUSY
WRINS_NOCHK
		BCF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTC
		BCF			E
		MOVLW		.255
		MOVWF		PORTC
		RETURN
CHK_BUSY
		BANKSEL		TRISC		;CHANGING OUTPUT TO INPUT
		MOVLW		.255
		MOVWF		TRISC		;LOAD
		BANKSEL		PORTC
		BCF			RS			;RS LOW
		BSF			RW			;RW HIGH
		BSF			E			;ENABLE HIGH
		CALL		DELAY30uS	;ADDED FOR BUSY CHECK
;		BTFSC		PORTC,7		;CHECK BUSY DOES NOT WORK
;		GOTO		$-1
		BANKSEL		TRISC	
		CLRF		TRISC		;LOAD MAKE OUTPUT
		BANKSEL		PORTA		;CHANGE BANKS
		BCF			E			;LOWER ENABLE
		RETURN					;RETURN


DELAY40MS
		MOVLW		.44
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN
DELAY30uS
		MOVLW		.11
		MOVWF		COUNTER0
		DECFSZ		COUNTER0,1
		GOTO		$-1
		RETURN
DELAY2MS
		MOVLW		3
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN

;***********************************INTIALIZING LCD DISPLAY*******************
DOG		CALL		DELAY40MS
		MOVLW		039H			;FUNCTION SET INSTRUCTION SET 0
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		01CH			;SET BIAS 3.3 VOLT 014H   5 VOLT 01CH
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		078H			;CONTRAST SET ADJUSTMENT 3.3 VOLT 078H  5 VOLT 074H USE 78 ON 4.5 VOLTS	
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		052H			;POWER CONTROL 3.3 VOLT 055H  5 VOLT 052H
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		069H			;FOLLOWER CONTROL 3.3 VOLT 06DH     5 VOLT 069H
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		038H			;FUNCTION SET  INSTRUCTION SET 1
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		00CH			;DISPLAY ON CURSOR OFF
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		001H			;CLEAR DISPLAY
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		002H			;GO HOME
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		006H			;ENTRY MODE SET
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS	
		RETURN


;**************************** DISPLAYING *****************************8

DISPLAYING	BTFSC	FLAG,2			;CHECK FOR VOLTAGE FLAG
		GOTO	DP1
		MOVLW	.0
		SUBWF	TENK,0
		BTFSS	3,2
		GOTO	DP1
		MOVLW	.10
		MOVWF	TENK
		
		MOVLW	.0				;PUTTING IN THE BLANKS
		SUBWF	THOUSANDS,0
		BTFSS	3,2
		GOTO	DP1
		MOVLW	.10

		MOVWF	THOUSANDS
		MOVLW	.0
		SUBWF	HUNDREDS,0
		BTFSS	3,2
		GOTO	DP1
		MOVLW	.10
		MOVWF	HUNDREDS

		MOVLW	.0
		SUBWF	TENS,0
		BTFSS	3,2
		GOTO	DP1
		MOVLW	.10
		MOVWF	TENS

		
DP1		MOVLW	0				;LOAD INTO 0 THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS
		ADDWF	ONES,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADRES)
		MOVLW	.9				;LOAD 3 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC	STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW	1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF	PCLATH
	 	MOVF 	ONES,W      	 ;MOVE CONTROL INTO W
	  	CALL    DISPLAY          ;GO TO LOOK UP TABLE       
		MOVWF	DIGIT_1
		CLRF	PCLATH

		MOVLW	0				;LOAD INTO 0 THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS
		ADDWF	TENS,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADRES)
		MOVLW	.9				;LOAD 3 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC	STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW	1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF	PCLATH
		MOVF 	TENS,W      	 ;MOVE CONTROL INTO W
		CALL    DISPLAY          ;GO TO LOOK UP TABLE       
		MOVWF	DIGIT_2
		CLRF	PCLATH

		MOVLW		0
		ADDWF		HUNDREDS,0
		MOVLW		.9
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		HUNDREDS,0
		CALL		DISPLAY
		MOVWF		DIGIT_3
		CLRF		PCLATH

		MOVLW		0
		ADDWF		THOUSANDS,0
		MOVLW		.9
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		THOUSANDS,0
		CALL		DISPLAY
		MOVWF		DIGIT_4
		CLRF		PCLATH

		MOVLW		0
		ADDWF		TENK,0
		MOVLW		.9
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		TENK,0
		CALL		DISPLAY
		MOVWF		DIGIT_5
		CLRF		PCLATH		
		RETURN

;____________________________________________________________________________



	
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     24-bit Addition                                                    -*
;*-                                                                        -*
;*-     Uses ACCa and ACCb                                                 -*
;*-                                                                        -*
;*-     ACCa + ACCb -> ACCb                                                -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Add24
        movf    ACCaLO,W
        addwf   ACCbLO,1          ;add low bytes
        btfsc   STATUS,C        ;add in carry if necessary
        goto    A2
A1      movf    ACCaMID,W
        addwf   ACCbMID,1         ;add mid bytes
        btfsc   STATUS,C        ;add in carry if necessary
        incf    ACCbHI,1
        movf    ACCaHI,W
        addwf   ACCbHI,1          ;add high bytes
        retlw   0
A2      incf    ACCbMID,1
        btfsc   STATUS,Z
        incf    ACCbHI,1
        goto    A1
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*       
;*-     Subtraction ( 24 - 24 -> 24 )                                      -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCd                                              -*
;*-                                                                        -*
;*-     ACCa -> ACCd,                                                      -*
;*-     2's complement ACCa,                                               -*
;*-     call Add24 ( ACCa + ACCb -> ACCb ),                                -*
;*-     ACCd -> ACCa                                                       -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Sub24
        movf    ACCaHI,W                ;Transfer ACCa to ACCd
        movwf   ACCdHI
        movf    ACCaMID,W
        movwf   ACCdMID
        movf    ACCaLO,W
        movwf   ACCdLO
        call    compA                   ;2's complement ACCa
        call    Add24                   ;Add ACCa to ACCb
        movf    ACCdHI,W                ;Transfer ACCd to ACCa
        movwf   ACCaHI
        movf    ACCdMID,W
        movwf   ACCaMID
        movf    ACCdLO,W
        movwf   ACCaLO
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Multiply ( 24 X 24 -> 56 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     ACCa * ACCb -> ACCb,ACCc  56-bit output                            -*
;*-     with ACCb (ACCbHI,ACCbMID,ACCbLO) with 24 msb's and                -*
;*-     ACCc (ACCcHI,ACCcMID,ACCcLO) with 24 lsb's                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Mpy24
        call    Msetup
mloop   rrf     ACCdHI,1          ;rotate d right
        rrf     ACCdMID,1
        rrf     ACCdLO,1
        btfsc   STATUS,C        ;need to add?
        call    Add24
        rrf     ACCbHI,1
        rrf     ACCbMID,1
        rrf     ACCbLO,1
        rrf     ACCcHI,1
        rrf     ACCcMID,1
        rrf     ACCcLO,1
        decfsz  temp,1            ;loop until all bits checked
        goto    mloop
        retlw   0

Msetup
        movlw   0x18            ;for 24 bit shifts
        movwf   temp
        movf    ACCbHI,W        ;move ACCb to ACCd
        movwf   ACCdHI
        movf    ACCbMID,W
        movwf   ACCdMID
        movf    ACCbLO,W
        movwf   ACCdLO
        clrf    ACCbHI
        clrf    ACCbMID
        clrf    ACCbLO
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Division ( 56 / 24 -> 24 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     56-bit dividend in ACCb,ACCc ( ACCb has msb's and ACCc has lsb's)  -*
;*-     24-bit divisor in ACCa                                             -*
;*-     quotient is stored in ACCc                                         -*
;*-     remainder is stored in ACCb                                        -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Div24
        call    Dsetup

dloop   bcf     STATUS,C
        rlf     ACCcLO,1         ;Rotate dividend left 1 bit position
        rlf     ACCcMID,1
        rlf     ACCcHI,1
        rlf     ACCbLO,1
        rlf     ACCbMID,1
        rlf     ACCbHI,1

        btfsc   STATUS,C        ;invert carry and exclusive or with the
        goto    clear           ;msb of the divisor then move this bit
        btfss   ACCaHI,msb_bit  ;into the lsb of the dividend
        incf    ACCcLO,1
        goto    cont
clear   btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1

cont    btfsc   ACCcLO,lsb_bit  ;check the lsb of the dividend
        goto    minus
        call    Add24           ;if = 0, then add divisor to upper 24 bits
        goto    check           ;of dividend
minus   call    Sub24           ;if = 1, then subtract divisor from upper
                                ;24 bits of dividend

check   decfsz  temp,f          ;do 24 times
        goto    dloop

        bcf     STATUS,C
        rlf     ACCcLO,1          ;shift lower 24 bits of dividend 1 bit 
        rlf     ACCcMID,1         ;position left
        rlf     ACCcHI,1
        btfsc   ACCbHI,msb_bit  ;exlusive or the inverse of the msb of the
        goto    w1              ;dividend with the msb of the divisor
        btfss   ACCaHI,msb_bit  ;store in the lsb of the dividend
        incf    ACCcLO,1
        goto    wzd
w1      btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1
wzd     btfss   ACCbHI,msb_bit  ;if the msb of the remainder is set and
        goto    wend
        btfsc   ACCaHI,msb_bit  ;the msb of the divisor is not
        goto    wend
        call    Add24           ;add the divisor to the remainder to correct
                                ;for zero partial remainder

wend    retlw   0               ;quotient in 24 lsb's of dividend
                                ;remainder in 24 msb's of dividend

Dsetup        
        movlw   0x18            ;loop 24 times
        movwf   temp
        
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     2's Complement                                                     -*
;*-                                                                        -*
;*-     Uses ACCa                                                          -*
;*-                                                                        -*
;*-     Performs 2's complement conversion on ACCa                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
compA    
        comf    ACCaLO,1          ;invert all bits in accumulator a
        comf    ACCaMID,1
        comf    ACCaHI,1
        incf    ACCaLO,1          ;add one to accumulator a
        btfsc   STATUS,Z
        incf    ACCaMID,1
        btfsc   STATUS,Z
        incf    ACCaHI,1
        retlw   0
;___________________________________________________
;------------------------------------------------------------------------------

CLEAR_REG
		CLRF	S1_TIMER_LO
		CLRF	S2_TIMER_LO
		CLRF	S3_TIMER_LO
		CLRF	S4_TIMER_LO
		CLRF	S5_TIMER_LO
		CLRF	S6_TIMER_LO
		CLRF	S7_TIMER_LO
		CLRF	S8_TIMER_LO
		CLRF	S9_TIMER_LO
		CLRF	S10_TIMER_LO
		CLRF	S1_TIMER_MID
		CLRF	S2_TIMER_MID
		CLRF	S3_TIMER_MID
		CLRF	S4_TIMER_MID
		CLRF	S5_TIMER_MID
		CLRF	S6_TIMER_MID
		CLRF	S7_TIMER_MID
		CLRF	S8_TIMER_MID
		CLRF	S9_TIMER_MID
		CLRF	S10_TIMER_MID
		CLRF	S1_TIMER_HI
		CLRF	S2_TIMER_HI
		CLRF	S3_TIMER_HI
		CLRF	S4_TIMER_HI
		CLRF	S5_TIMER_HI
		CLRF	S6_TIMER_HI
		CLRF	S7_TIMER_HI
		CLRF	S8_TIMER_HI
		CLRF	S9_TIMER_HI
		CLRF	S10_TIMER_HI
		CLRF	AVG_TIMER_LO
		CLRF	AVG_TIMER_MID
		CLRF	AVG_TIMER_HI
		CLRF	MAX_L
		CLRF	MAX_H
		CLRF	MAX_TIMER_HI
		CLRF	MIN_L
		CLRF	MIN_H
		CLRF	MIN_TIMER_HI
		MOVLW	031H
		MOVWF	SHOT
		RETURN

;********************USED FOR TROUBLE SHOOTING******************
CHECK
		CALL		DOG
		BANKSEL		PORTA
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		048H		;H
		CALL		SEND
		MOVLW		045H		;E
		CALL		SEND
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		04BH		;K
		CALL		SEND
		GOTO	$

;******************************LOOK UP TABLE FOR NUMBERS*********************
		ORG  0x900
DISPLAY
		ADDWF	PCL,1				;ADD W TO PROGRAM COUNTER AND LOOK UP DATA  "." = DECIMAL #
		RETLW	B'00110000'			;0
		RETLW	B'00110001'			;1
		RETLW	B'00110010'			;2
		RETLW	B'00110011'			;3
		RETLW	B'00110100'			;4
		RETLW	B'00110101'			;5
		RETLW	B'00110110'			;6
		RETLW	B'00110111'			;7
		RETLW	B'00111000'			;8
		RETLW	B'00111001'			;9
		RETLW	020H				; BLANK
				RETURN

VOLT1		MOVLW	B'10000101'	;RA1 VOLTAGE
		MOVWF	ADCON0
		BSF	ADCON0,GO
		BTFSC	ADCON0,GO
		GOTO	$-1
		PAGESEL	START
		CALL	CLEAR

		
		MOVF	ADRESH,0
		MOVWF	ACCaMID
		BANKSEL	ADRESL
		MOVF	ADRESL,0
		BANKSEL	PORTA
		MOVWF	ACCaLO	    
		MOVLW	0E8H
		MOVWF	ACCbLO
		MOVLW	01H
		MOVWF	ACCbMID
		PAGESEL	START
		CALL	Mpy24		;MULTIPLY BY 488
		MOVLW	0E8H
		MOVWF	ACCaLO
		MOVLW	03H
		MOVWF	ACCaMID
		MOVLW	0H
		MOVWF	ACCaHI
		PAGESEL	START
		CALL	Div24		;DIVIDE BY 100,000
		PAGESEL	START
		CALL	DECIMAL
		PAGESEL	START
		CALL	DISPLAYING
		
		PAGESEL	START
		CALL	DOG
		MOVLW	053H			;START
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	03DH			;=
		CALL	SEND

		MOVF	DIGIT_3,0		;
		CALL	SEND
		MOVLW	02EH			;DECIMAL
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	056H			;VOLTS
		CALL	SEND
		MOVLW	04FH			;0
		CALL	SEND
		MOVLW	04CH			;L
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND
		PAGESEL	VOLT1
		MOVLW	B'10001001'	;RA2 VOLTAGE
		MOVWF	ADCON0
		BSF	ADCON0,GO
		BTFSC	ADCON0,GO
		GOTO	$-1
		PAGESEL	START
		CALL	CLEAR
		MOVF	ADRESH,0
		MOVWF	ACCaMID
		BANKSEL	ADRESL
		MOVF	ADRESL,0
		BANKSEL	PORTA
		MOVWF	ACCaLO	    
		MOVLW	0E8H
		MOVWF	ACCbLO
		MOVLW	01H
		MOVWF	ACCbMID
		PAGESEL	Mpy24
		CALL	Mpy24		;MULTIPLY BY 488
		MOVLW	0EAH
		MOVWF	ACCaLO
		MOVLW	03H
		MOVWF	ACCaMID
		MOVLW	0H
		MOVWF	ACCaHI
		CALL	Div24		;DIVIDE BY 100,000
		PAGESEL	START
		CALL	DECIMAL
		PAGESEL	START
		CALL	DISPLAYING		
		
		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		PAGESEL	START
		CALL	WRINS_CHK	
		PAGESEL	START
		MOVLW	053H			;STOP
		CALL	SEND	
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	050H			;P
		CALL	SEND	
		MOVLW	03DH			;=
		CALL	SEND	
		MOVF	DIGIT_3,0		
		CALL	SEND
		MOVLW	02EH			;DECIMAL
		CALL	SEND
		MOVF	DIGIT_2,0
		CALL	SEND
		MOVF	DIGIT_1,0
		CALL	SEND
		MOVLW	020H
		CALL	SEND
		MOVLW	056H			;VOLTS
		CALL	SEND
		MOVLW	04FH			;0
		CALL	SEND
		MOVLW	04CH			;L
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND		
		PAGESEL	VOLT1
		MOVLW	.5
		MOVWF	COUNT2
		DECFSZ	COUNT,1
		GOTO	$-1
		DECFSZ	COUNT1,1
		GOTO	$-3
		DECFSZ	COUNT2,1
		GOTO	$-5
		PAGESEL	VOLT1
		GOTO	VOLT1
  END